"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("mz/fs");
const nsfw = require("nsfw");
const path = require("path");
const utility_1 = require("../utility");
const Language_1 = require("./Language");
const RBX_SEPARATOR = ".";
const FILE_TYPE_EXTENSIONS = [
    ["client", "LocalScript"],
    ["", "ModuleScript"],
    ["server", "Script"],
];
function getTypeBySubExtension(ext) {
    for (const pair of FILE_TYPE_EXTENSIONS) {
        if (pair[0] === ext) {
            return pair[1];
        }
    }
    return null;
}
function filterNsfwEvents(curr, index, events) {
    if (index > 0) {
        const prev = events[index - 1];
        if (curr.action === prev.action) {
            if (curr.action === nsfw.actions.RENAMED && prev.action === nsfw.actions.RENAMED) {
                if (curr.directory === prev.directory &&
                    curr.newDirectory === prev.newDirectory &&
                    curr.newFile === prev.newFile &&
                    curr.oldFile === prev.oldFile) {
                    return false;
                }
            }
            else if (curr.action !== nsfw.actions.RENAMED && prev.action !== nsfw.actions.RENAMED) {
                if (curr.directory === prev.directory && curr.file === prev.file) {
                    return false;
                }
            }
        }
    }
    return true;
}
class Partition {
    constructor(project, name, directory, target) {
        this.project = project;
        this.name = name;
        this.directory = directory;
        this.target = target;
        this.isRunning = false;
        this.isSingleFile = false;
        this.isSingleFileSet = false;
        this.rbxPath = target.split(RBX_SEPARATOR);
        this.setSingleFile();
    }
    setSingleFile() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isSingleFileSet) {
                this.isSingleFile = (yield fs.stat(this.directory)).isFile();
                this.isSingleFileSet = true;
            }
        });
    }
    addChangesRecursive(changes = new Array(), dir = this.directory) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setSingleFile();
            if (this.isSingleFile) {
                changes.push(this.getChangeFromFile(dir));
            }
            else {
                const promises = new Array();
                for (const fileName of yield fs.readdir(dir)) {
                    const filePath = path.resolve(dir, fileName);
                    if ((yield fs.stat(filePath)).isDirectory()) {
                        promises.push(this.addChangesRecursive(changes, filePath));
                    }
                    else if (filePath.match(Language_1.default.ignoreRegExp) === null) {
                        changes.push(this.getChangeFromFile(filePath));
                    }
                }
                yield Promise.all(promises);
            }
        });
    }
    getUpdateFromFile(filePath) {
        const fileExt = path.extname(filePath);
        const fullName = path.basename(filePath, fileExt);
        const fileTypeExt = path
            .extname(fullName)
            .substr(1) // remove leading period
            .toLowerCase();
        let changePath = this.rbxPath.concat(path.relative(this.directory, filePath).split(path.sep));
        changePath.pop();
        changePath = changePath
            .filter(value => value.length > 0)
            .reduce((accum, value) => accum.concat(...value.split(".")), new Array());
        let changeType = fileExt === "" ? "Folder" : getTypeBySubExtension(fileTypeExt.toLowerCase());
        let fileName = path.basename(fullName, "." + fileTypeExt);
        if (!changeType) {
            changeType = "ModuleScript";
            fileName += "." + fileTypeExt;
        }
        if (!this.isSingleFile) {
            changePath.push(fileName);
        }
        return {
            path: changePath,
            type: changeType,
        };
    }
    getRemoveFromFile(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const update = this.getUpdateFromFile(filePath);
            return {
                path: update.path,
                source: null,
                type: update.type,
            };
        });
    }
    getChangeFromFile(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const update = this.getUpdateFromFile(filePath);
            const ext = path.extname(filePath);
            let language;
            for (const lang of Language_1.default.instances) {
                if (ext === lang.ext) {
                    language = lang;
                }
            }
            let changeSource;
            if (language) {
                changeSource = (yield language.getSource(filePath)).toString();
            }
            else {
                changeSource = (yield utility_1.getFileContents(filePath)).toString();
            }
            return {
                path: update.path,
                source: changeSource,
                type: update.type,
            };
        });
    }
    syncChangeToStudio(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            this.project.distributeChangeToStudio(yield this.getChangeFromFile(filePath));
        });
    }
    syncRemoveToStudio(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            this.project.distributeChangeToStudio(yield this.getRemoveFromFile(filePath));
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isRunning) {
                this.isRunning = true;
                console.log("start", "partition", this.name, path.relative(this.project.directory, this.directory));
                this.watcher = yield nsfw(this.directory, (events) => __awaiter(this, void 0, void 0, function* () {
                    events = events.filter(filterNsfwEvents);
                    // replace RENAMED with DELETED -> CREATED
                    for (let i = 0; i < events.length; i++) {
                        const event = events[i];
                        if (event.action === nsfw.actions.RENAMED) {
                            const deletedEvent = {
                                action: nsfw.actions.DELETED,
                                directory: event.directory,
                                file: event.oldFile,
                            };
                            const createdEvent = {
                                action: nsfw.actions.CREATED,
                                directory: event.directory,
                                file: event.oldFile,
                            };
                            events.splice(i, 1, deletedEvent, createdEvent);
                        }
                    }
                    for (const event of events) {
                        if (event.action === nsfw.actions.RENAMED) {
                            throw new Error("Unexpected RENAMED event!");
                        }
                        const filePath = path.join(event.directory, event.file);
                        const isFile = (yield fs.exists(filePath)) && (yield fs.lstat(filePath)).isFile();
                        if (event.action === nsfw.actions.CREATED && isFile) {
                            yield this.syncChangeToStudio(filePath);
                        }
                        else if (event.action === nsfw.actions.MODIFIED && isFile) {
                            yield this.syncChangeToStudio(filePath);
                        }
                        else if (event.action === nsfw.actions.DELETED) {
                            yield this.syncRemoveToStudio(filePath);
                        }
                    }
                }));
                yield this.watcher.start();
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isRunning) {
                this.isRunning = false;
                console.log("stop watch", "partition", this.name, path.relative(this.project.directory, this.directory));
                if (this.watcher) {
                    yield this.watcher.stop();
                    this.watcher = undefined;
                }
            }
        });
    }
}
exports.default = Partition;
//# sourceMappingURL=Partition.js.map