"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("mz/fs");
const nsfw = require("nsfw");
const path = require("path");
const util = require("util");
const PathReporter_1 = require("io-ts/lib/PathReporter");
const types_1 = require("../types");
const utility_1 = require("../utility");
const Client_1 = require("./Client");
const Partition_1 = require("./Partition");
const CONFIG_FILE_NAMES = ["rofresh.json", "rojo.json"];
const DEFAULT_PARTITIONS = {
    default: {
        path: "src",
        target: "",
    },
};
class Project {
    constructor(directory) {
        this.isRunning = false;
        this.allowAnyPlaceId = false;
        this._placeIds = new Set();
        this.placeIds = this._placeIds;
        this.partitions = new Array();
        this.name = "";
        this.loaded = false;
        Project._instances.push(this);
        this.directory = path.resolve(directory);
        this.setup();
    }
    getConfigPath() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const name of CONFIG_FILE_NAMES) {
                const filePath = path.join(this.directory, name);
                if (yield fs.exists(filePath)) {
                    return filePath;
                }
            }
            this.remove();
        });
    }
    setup() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!(yield fs.exists(this.directory))) {
                throw new Error(util.format("Could not find project directory! [ %s ]", this.directory));
            }
            const configPath = yield this.getConfigPath();
            console.log("configPath", configPath);
            if (configPath) {
                this.configWatcher = yield nsfw(configPath, events => {
                    for (const event of events) {
                        if (event.action === nsfw.actions.MODIFIED) {
                            this.readConfig(configPath);
                        }
                        else if (event.action === nsfw.actions.DELETED) {
                            this.remove();
                        }
                    }
                });
                yield this.configWatcher.start();
                yield this.readConfig(configPath);
            }
        });
    }
    remove() {
        return __awaiter(this, void 0, void 0, function* () {
            const index = Project.instances.indexOf(this);
            if (index > -1) {
                Project._instances.splice(index, 1);
            }
            // cleanup
            yield this.stop();
            if (this.configWatcher) {
                yield this.configWatcher.stop();
                this.configWatcher = undefined;
            }
        });
    }
    isValidPlaceId(placeId) {
        return this.allowAnyPlaceId || this.placeIds.has(placeId);
    }
    applyConfig(config) {
        return __awaiter(this, void 0, void 0, function* () {
            let restartAll = false;
            this.name = config.name;
            // normalize falsey -> false
            if (config.allowAnyPlaceId === undefined) {
                config.allowAnyPlaceId = false;
            }
            restartAll = restartAll || config.allowAnyPlaceId !== this.allowAnyPlaceId;
            this.allowAnyPlaceId = config.allowAnyPlaceId === true;
            // is this still necessary?
            if (!this.allowAnyPlaceId && config.placeIds) {
                for (const placeId of this.placeIds) {
                    if (config.placeIds.indexOf(placeId) === -1) {
                        this._placeIds.delete(placeId);
                    }
                }
                for (const placeId of config.placeIds) {
                    if (!this.placeIds.has(placeId)) {
                        this._placeIds.add(placeId);
                    }
                }
            }
            const configPartitions = new Array();
            // resolve paths
            const partitionMap = config.partitions || DEFAULT_PARTITIONS;
            for (const name in partitionMap) {
                configPartitions.push({
                    name,
                    path: path.resolve(this.directory, partitionMap[name].path),
                    target: partitionMap[name].target,
                });
            }
            // remove old partitions
            for (let i = this.partitions.length - 1; i >= 0; i--) {
                const part = this.partitions[i];
                let found = false;
                for (const info of configPartitions) {
                    if (part.name === info.name && part.directory === info.path && part.target === info.target) {
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    this.partitions.splice(i, 1);
                    part.stop();
                }
            }
            // add new partitions
            for (const info of configPartitions) {
                let found = false;
                for (const part of this.partitions) {
                    if (part.name === info.name && part.directory === info.path && part.target === info.target) {
                        found = true;
                        break;
                    }
                }
                if (!found && (yield fs.exists(info.path))) {
                    const partition = new Partition_1.default(this, info.name, info.path, info.target);
                    this.partitions.push(partition);
                    if (this.isRunning) {
                        yield partition.start();
                    }
                }
            }
            this.loaded = true;
            if (this.isRunning) {
                Client_1.default.instances
                    .filter(client => this.isValidPlaceId(client.placeId))
                    .forEach(client => client.fullSyncProjectToStudio(this));
            }
        });
    }
    readConfig(configPath) {
        return __awaiter(this, void 0, void 0, function* () {
            let configJson;
            if (yield fs.exists(configPath)) {
                const fileContents = yield utility_1.getFileContents(configPath);
                try {
                    configJson = JSON.parse(fileContents.toString());
                }
                catch (e) {
                    console.log(util.format("Could not parse JSON ( %s )", configPath));
                    return;
                }
            }
            else {
                // no config, no project
                this.remove();
                return;
            }
            const configIO = types_1.RofreshConfigIO.decode(configJson).map(config => this.applyConfig(config));
            if (configIO.isLeft()) {
                console.warn("Config Error", PathReporter_1.PathReporter.report(configIO));
            }
        });
    }
    distributeChangeToStudio(change) {
        return __awaiter(this, void 0, void 0, function* () {
            Client_1.default.instances
                .filter(client => this.isValidPlaceId(client.placeId))
                .forEach(client => client.syncToStudio(this.name, [change]));
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isRunning) {
                console.log("start", "project", this.directory);
                this.isRunning = true;
                for (const partition of this.partitions) {
                    console.log("part", partition.name);
                    yield partition.start();
                }
                Client_1.default.instances
                    .filter(client => this.isValidPlaceId(client.placeId))
                    .forEach(client => client.fullSyncProjectToStudio(this));
            }
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isRunning) {
                console.log("stop", "project", this.directory);
                this.isRunning = false;
                for (const partition of this.partitions) {
                    yield partition.stop();
                }
            }
        });
    }
}
Project._instances = new Array();
Project.instances = Project._instances;
exports.default = Project;
//# sourceMappingURL=Project.js.map